/*
 * Controller.java
 *
 * Created on March 7, 2006, 1:23 PM
 */

package com.runstate.licman;

import com.runstate.licman.entities.Application;
import com.runstate.licman.entities.Licence;
import com.runstate.licman.entities.User;
import com.runstate.licman.entities.Version;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;

/**
 *
 * @author dj
 */
public class Controller {
    
    LicManUI ui;
    
    LicManStore licmanStore;
    
    Application currentApplication=null;
    Version currentVersion=null;
    Licence currentLicence=null;
    
    DefaultListModel appmodel=new DefaultListModel();
    DefaultListModel versionmodel=new DefaultListModel();
    DefaultListModel licencemodel=new DefaultListModel();
    DefaultListModel usermodel=new DefaultListModel();
    DefaultListModel allusermodel=new DefaultListModel();
    
    /**
     * Creates a new instance of Controller
     */
    public Controller() {
        licmanStore=LicManStore.getStore();
        populateApplications();
        populateAllUsers();
        ui=new LicManUI(this);
        ui.setVersionPaneEnabled(false);
        ui.setLicencePaneEnabled(false);
        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                ui.setVisible(true);
            }
        });
    }
    
    public ListModel getApplicationListModel() { return appmodel; }
    public ListModel getVersionListModel() { return versionmodel; }
    public ListModel getLicenceListModel() { return licencemodel; }
    public ListModel getUserListModel() { return usermodel; }
    public ListModel getAllUserListModel() { return allusermodel; }
    
    void selectedApplication(Application app) {
        if(app==currentApplication) return;
        currentApplication=app;
        populateVersions();
    }
    void selectedVersion(Version version) {
        if(version==currentVersion) return;
        currentVersion=version;
        populateLicences();
        ui.setLicencePaneEnabled(currentVersion!=null);
    }
    void selectedLicence(Licence licence) {
        if(licence==currentLicence) return;
        currentLicence=licence;
        populateLicenceUsers();
        ui.setUserPaneEnabled(currentLicence!=null);
    }
    
    
    private void populateApplications() {
        appmodel.removeAllElements();
        List<Application> apps=licmanStore.getApplications();
        for(Application a:apps) { appmodel.addElement(a); }
    }
    
    private void populateAllUsers() {
        allusermodel.removeAllElements();
        List<User> allusers=licmanStore.getAllUsers();
        for(User u:allusers) { allusermodel.addElement(u); }
    }
    
    private void populateVersions() {
        versionmodel.removeAllElements();
        if(currentApplication!=null) {
            for(Version v:currentApplication.getVersions()) {
                versionmodel.addElement(v);
            }
            ui.setVersionPaneEnabled(true);
        } else {
            ui.setVersionPaneEnabled(false);
        }
    }
    
    private void populateLicences() {
        licencemodel.removeAllElements();
        if(currentVersion!=null) {
            for(Licence l:currentVersion.getLicences()) {
                licencemodel.addElement(l);
            }
        }
    }
    
    
    void populateLicenceUsers() {
        usermodel.removeAllElements();
        if(currentLicence!=null) {
            for(User u:currentLicence.getUsers()) {
                usermodel.addElement(u);
            }
        }
    }
    
    void addApplication(String appname) throws UpdateException {
        if(licmanStore.getApplication(appname)!=null)
            throw new UpdateException("An application "+appname+" already exists");
        Application app=new Application();
        app.setApplicationname(appname);
        licmanStore.saveApplication(app);
        populateApplications();
        ui.setSelectedApplication(app);
    }
    
    void addVersion(String versionname) throws UpdateException {
        if(currentApplication==null) return;
        if(licmanStore.getVersion(currentApplication,versionname)!=null) 
            throw new UpdateException("Version "+versionname+" already exists");
        Version ver=new Version();
        ver.setVersionname(versionname);
        ver.setApplication(currentApplication);
        currentApplication.getVersions().add(ver);
        licmanStore.updateApplication(currentApplication);
        populateVersions();
        ver=licmanStore.getVersion(currentApplication,versionname);
        ui.setSelectedVersion(ver);
    }
    
   
    void addLicence(String s) {
        if(currentVersion==null) return;
        Licence lic=new Licence();
        lic.setLicenceKey(s);
        lic.setVersion(currentVersion);
        currentVersion.getLicences().add(lic);
        licmanStore.updateApplication(currentApplication);
        populateLicences();
        ui.setSelectedLicence(lic);
    }
    
    void addUser(String s) throws UpdateException {
        User testuser=licmanStore.getUser(s);
        if(testuser!=null) throw new UpdateException("User "+s+" already exists");
        
        User user=new User();
        user.setUserName(s);
        licmanStore.saveUser(user);
        populateAllUsers();
    }
    
    void addUsersToLicence(List<User> u) throws UpdateException {
        if(currentLicence==null) throw new UpdateException("No licence selected");
        currentLicence.getUsers().addAll(u);
        licmanStore.updateApplication(currentApplication);
        for(User ut:u) licmanStore.refresh(ut);
        populateLicenceUsers();
    }
    
    void removeUsersFromLicence(List<User> u) throws UpdateException {
        if(currentLicence==null) throw new UpdateException("No licence selected");
        currentLicence.getUsers().removeAll(u);
        licmanStore.updateApplication(currentApplication);
        for(User ut:u) licmanStore.refresh(ut);
        populateLicenceUsers();
    }
    
    void deleteApplication(Application app) {
        licmanStore.deleteApplication(app);
        appmodel.removeElement(app);
        currentApplication=null;
        populateVersions();
        populateLicences();
        populateLicenceUsers();
    }
    
    void deleteLicence(Licence l) {
        currentVersion.getLicences().remove(l);
        currentLicence=null;
        licmanStore.updateApplication(currentApplication);
        populateLicences();
        populateLicenceUsers();
    }
    
    void deleteVersion(Version v) {
        currentApplication.getVersions().remove(v);
        currentVersion=null;
        currentLicence=null;
        licmanStore.updateApplication(currentApplication);
        populateVersions();
        populateLicences();
        populateLicenceUsers();
    }
    
    void showUserReport(User user) {
        StringBuilder sb=new StringBuilder();
        
        sb.append(user.getUserName());
        if(user.getLicences().size()==0) {
            sb.append(" has no licences");
        } else {
            int liccount=user.getLicences().size();
            sb.append(" has "+liccount+" licence"+(liccount>1?"s":"")+"\n");
            
            for(Licence l:user.getLicences()) {
                sb.append(l.getVersion().getApplication().getApplicationname());
                sb.append(" ");
                sb.append(l.getVersion().getVersionname());
                sb.append(" ");
                sb.append(l.getLicenceKey());
                sb.append("\n");
            }
        }
        UserReportUI urui=new UserReportUI(sb.toString());
        urui.setVisible(true);
    }
    
}
